import fedmsg
import fedmsg.encoding
import moksha.hub.reactor

from twisted.internet.task import LoopingCall

from . import utils, config
from . import JobTrigger

class BodhiJobTrigger(JobTrigger):
    topic = "org.fedoraproject.prod.bodhi.update.request.*"
    jsonify = False
    config_key = 'taskotron.bodhijobtrigger.enabled'

    def __init__(self, *args, **kw):
        super(BodhiJobTrigger, self).__init__(*args, **kw)

        self.queued_tags = set()
        self.queued_arches = set()

        # check for bodhi_to_koji_tags tasks queue every config.fuse_delay seconds
        lc = LoopingCall(self.delayed_consume)
        lc.start(config.fuse_delay)

    def delayed_consume(self):
        if self.queued_tags:
            try:
                for tag in self.queued_tags:
                    self.trigger_tasks(tag, 'koji_tag', config.bodhi_to_koji_tag_tasks, self.queued_arches)
            finally:
                self.queued_tags = set()
                self.queued_arches = set()
        else:
            self.log.debug("Woke up, but there were no messages.")

    def consume(self, message):
        usable_message = fedmsg.encoding.loads(message.body)

        topic = usable_message['topic']
        builds = usable_message['msg']['update']['builds']
        nvrs = [build['nvr'] for build in builds]

        # we want only requests to stable and testing
        if not topic.endswith('stable') and not topic.endswith('testing'):
            return

        # do not schledule jobs for blacklisted releases
        if nvrs[0].endswith(tuple(config.blacklist_releases)):
            return

        updatetitle = usable_message['msg']['update']['title']
        arches = utils.get_arches(nvr=nvrs[0])

        # XXX T252
        # only schedule koji_tag tasks for -pending tags
        if usable_message['msg']['update']['status'] == 'pending':
            dist_tag = usable_message['msg']['update']['release']['dist_tag']

            koji_tag = '%s-updates' % dist_tag
            if topic.endswith('testing'):
                koji_tag += '-testing'
            koji_tag += '-pending'

            if not len(self.queued_tags):
                self.log.info('setting fuse (%d seconds) for koji_tag %s', config.fuse_delay, koji_tag)
            else:
                self.log.info('adding koji_tag %s to the fuse queue', koji_tag)

            self.queued_tags.add(koji_tag)
            for arch in arches:
                self.queued_arches.add(arch)

        # run bodhi_id tasks for every update
        self.trigger_tasks(updatetitle, 'bodhi_id', config.bodhi_tasks, arches)
